import typer
import logging
from typer_htcondor import dagman

app = typer.Typer()


@app.command()
def submit(
    submit_file: str = typer.Argument(..., help="Submit file"),
    resource: str = typer.Option(
        None, "--resource", help="Resource to run this job. Supports Slurm and EC2"),
    runtime: int = typer.Option(
        None, "--runtime", help="Runtime for the given resource (seconds)"),
    email: str = typer.Option(
        None, "--email", help="Email address to receive notifications"),
    annex_name: str = typer.Option(
        None, "--annex-name", help="Annex name that this job must run on"),
):
    """
    Submits a job when given a submit file.
    """
    # Your command implementation goes here. @TODO
    print(f"Submitting job with submit file: {submit_file}")
    if resource:
        print(f"Resource: {resource}")
    if runtime is not None:
        print(f"Runtime: {runtime}")
    if email:
        print(f"Email: {email}")
    if annex_name:
        print(f"Annex name: {annex_name}")


@app.command()
def status(
    job_id: str = typer.Argument(..., help="Job ID"),
    skip_history: bool = typer.Option(
        False, "--skip-history", help="Skip checking history for completed or removed jobs"),
):
    """
    Shows current status of a job when given a job id.
    """

    # Split job id into cluster and proc
    if "." in job_id:
        cluster_id, proc_id = [int(i) for i in job_id.split(".")][:2]
    else:
        cluster_id = int(job_id)
        proc_id = 0

    # Get schedd
    schedd = htcondor.Schedd()

    # Query schedd
    constraint = f"ClusterId == {cluster_id} && ProcId == {proc_id}"
    projection = ["JobStartDate", "JobStatus", "QDate", "CompletionDate", "EnteredCurrentStatus",
                  "RequestMemory", "MemoryUsage", "RequestDisk", "DiskUsage", "HoldReason",
                  "ResourceType", "TargetAnnexName", "NumShadowStarts", "NumJobStarts", "NumHolds",
                  "JobCurrentStartTransferOutputDate", "TotalSuspensions", "NumRestarts", "CommittedTime"]
    try:
        job = schedd.query(constraint=constraint,
                           projection=projection, limit=1)
    except IndexError:
        raise RuntimeError(f"No job found for ID {job_id}.")
    except Exception as e:
        raise RuntimeError(f"Error looking up job status: {str(e)}")

    # Check the history if no jobs found
    if len(job) == 0 and not skip_history:
        try:
            job = list(schedd.history(constraint=constraint,
                       projection=projection, match=1))
        except KeyboardInterrupt:
            job = []

    if len(job) == 0:
        raise RuntimeError(f"No job found for ID {job_id}.")

    resource_type = job[0].get("ResourceType", "htcondor") or "htcondor"
    resource_type = resource_type.casefold()

    if resource_type == "htcondor":

        job_ad = job[0]

        try:
            job_status = job_ad["JobStatus"]
        except IndexError as err:
            logger.error(f"Job {job_id} has unknown status")
            raise err

        # Get annex info
        annex_info = ""
        target_annex_name = job_ad.get('TargetAnnexName')
        if target_annex_name is not None:
            annex_info = f" on the annex named '{target_annex_name}'"

        # Get some common numbers
        job_queue_time = datetime.now(
        ) - datetime.fromtimestamp(job_ad["QDate"])
        job_atts = job_ad.get("NumShadowStarts", 0)
        job_execs = job_ad.get("NumJobStarts", 0)
        job_holds = job_ad.get("NumHolds", 0)
        job_suspends = job_ad.get("TotalSuspensions", 0)
        job_committed_time = job_ad.get("CommittedTime")
        goodput = None
        if job_committed_time is not None:
            goodput = job_committed_time / 3600

        # Compute memory and disk usage if available
        memory_usage, disk_usage = None, None
        if "MemoryUsage" in job_ad:
            memory_usage = (f"{readable_size(job_ad.eval('MemoryUsage')*10**6)} out of "f"{readable_size(job_ad.eval('RequestMemory') * 10**6)} requested")
        if "DiskUsage" in job_ad:
            disk_usage = (f"{readable_size(job_ad.eval('DiskUsage')*10**3)} out of "f"{readable_size(job_ad.eval('RequestDisk')*10**3)} requested")

        # Print information relevant to each job status
        if job_status == htcondor.JobStatus.IDLE:
            logger.info(f"Job {job_id} is currently idle.")
            logger.info(f"It was submitted {readable_time(job_queue_time.seconds)} ago.")
            logger.info(f"It requested {readable_size(
                job_ad.eval('RequestMemory')*10**6)} of memory.")
            logger.info(f"It requested {readable_size(
                job_ad.eval('RequestDisk')*10**3)} of disk space.")
            if job_holds > 0:
                logger.info(f"It has been held {job_holds} time{s(job_holds)}.")
            if job_atts > 0:
                logger.info(f"HTCondor has attempted to start the job {job_atts} time{s(job_atts)}.")
                logger.info(f"The job has started {job_execs} time{s(job_execs)}.")

        elif job_status == htcondor.JobStatus.RUNNING:
            job_running_time = datetime.now(
            ) - datetime.fromtimestamp(job_ad["JobStartDate"])
            logger.info(f"Job {job_id} is currently running{annex_info}.")
            logger.info(f"It started running {readable_time(job_running_time.seconds)} ago.")
            logger.info(f"It was submitted {readable_time(job_queue_time.seconds)} ago.")
            if memory_usage:
                logger.info(f"Its current memory usage is {memory_usage}.")
            if disk_usage:
                logger.info(f"Its current disk usage is {disk_usage}.")
            if job_suspends > 0:
                logger.info(f"It has been suspended {job_suspends} time{s(job_suspends)}.")
            if job_holds > 0:
                logger.info(f"It has been held {job_holds} time{s(job_holds)}.")
            if job_atts > 1:
                logger.info(f"HTCondor has attempted to start the job {job_atts} time{s(job_atts)}.")
                logger.info(f"The job has started {job_execs} time{s(job_execs)}.")

        elif job_status == htcondor.JobStatus.SUSPENDED:
            job_suspended_time = datetime.now(
            ) - datetime.fromtimestamp(job_ad["EnteredCurrentStatus"])
            logger.info(f"Job {job_id} is currently suspended{annex_info}.")
            logger.info(f"It has been suspended for {readable_time(job_suspended_time.seconds)}.")
            logger.info(f"It has been suspended {job_suspends} time{s(job_suspends)}.")
            logger.info(f"It was submitted {readable_time(job_queue_time.seconds)} ago.")
            if memory_usage:
                logger.info(f"Its last memory usage was {memory_usage}.")
            if disk_usage:
                logger.info(f"Its last disk usage was {disk_usage}.")

        elif job_status == htcondor.JobStatus.TRANSFERRING_OUTPUT:
            job_transfer_time = datetime.now(
            ) - datetime.fromtimestamp(job_ad["JobCurrentStartTransferOutputDate"])
            logger.info(
                f"Job {job_id} is currently transferring output{annex_info}.")
            logger.info(f"It started transferring output {readable_time(job_transfer_time.seconds)} ago.")
            if memory_usage:
                logger.info(f"Its last memory usage was {memory_usage}.")
            if disk_usage:
                logger.info(f"Its last disk usage was {disk_usage}.")

        elif job_status == htcondor.JobStatus.HELD:
            job_held_time = datetime.now(
            ) - datetime.fromtimestamp(job_ad["EnteredCurrentStatus"])
            logger.info(f"Job {job_id} is currently held.")
            logger.info(f"It has been held for {readable_time(job_held_time.seconds)}.")
            logger.info(f"""It was held because "{job_ad['HoldReason']}".""")
            logger.info(f"It has been held {job_holds} time{s(job_holds)}.")
            logger.info(f"It was submitted {readable_time(job_queue_time.seconds)} ago.")
            if job_execs >= 1:
                if memory_usage:
                    logger.info(f"Its last memory usage was {memory_usage}.")
                if disk_usage:
                    logger.info(f"Its last disk usage was {disk_usage}.")
            if job_atts >= 1:
                logger.info(f"HTCondor has attempted to start the job {job_atts} time{s(job_atts)}.")
                logger.info(f"The job has started {job_execs} time{s(job_execs)}.")

        elif job_status == htcondor.JobStatus.REMOVED:
            job_removed_time = datetime.now(
            ) - datetime.fromtimestamp(job_ad["EnteredCurrentStatus"])
            logger.info(f"Job {job_id} was removed.")
            logger.info(f"It was removed {readable_time(
                job_removed_time.seconds)} ago.")
            logger.info(f"It was submitted {readable_time(job_queue_time.seconds)} ago.")
            if memory_usage:
                logger.info(f"Its last memory usage was {memory_usage}.")
            if disk_usage:
                logger.info(f"Its last disk usage was {disk_usage}.")

        elif job_status == htcondor.JobStatus.COMPLETED:
            job_completed_time = datetime.now(
            ) - datetime.fromtimestamp(job_ad["CompletionDate"])
            logger.info(f"Job {job_id} has completed.")
            logger.info(f"It completed {readable_time(
                job_completed_time.seconds)} ago.")
            logger.info(f"It was submitted {readable_time(job_queue_time.seconds)} ago.")
            if memory_usage:
                logger.info(f"Its last memory usage was {memory_usage}.")
            if disk_usage:
                logger.info(f"Its last disk usage was {disk_usage}.")
            if job_committed_time is not None:
                goodput = job_committed_time / 3600
                logger.info(f"Goodput is {goodput:.1f} hours.")
        else:
            logger.info(
                f"Job {job_id} is in an unknown state (JobStatus = {job_status}).")

    # Jobs running on provisioned Slurm or EC2 resources need to retrieve
    # additional information from the provisioning DAGMan log
    elif resource_type in ["slurm", "ec2"]:

        # Variables specific to jobs running on Slurm clusters
        jobs_running = 0
        job_started_time = None
        provisioner_cluster_id = None
        provisioner_job_submitted_time = None
        slurm_cluster_id = None
        slurm_nodes_requested = None
        slurm_runtime = None

        dagman_dag, dagman_out, dagman_log = DAGMan.get_files(job_id)

        if dagman_dag is None:
            raise RuntimeError(
                f"No {resource_type} job found for ID {job_id}.")

        # Parse the .dag file to retrieve some user input values
        with open(dagman_dag, "r") as dagman_dag_file:
            for line in dagman_dag_file.readlines():
                if "annex_runtime =" in line:
                    slurm_runtime = int(line.split("=")[1].strip())

        # Parse the DAGMan event log for useful information
        dagman_events = htcondor.JobEventLog(dagman_log)
        for event in dagman_events.events(0):
            if "LogNotes" in event.keys() and event["LogNotes"] == "DAG Node: B":
                provisioner_cluster_id = event.cluster
                provisioner_job_submitted_time = datetime.fromtimestamp(
                    event.timestamp)
                job_status = "PROVISIONING REQUEST PENDING"
            elif "LogNotes" in event.keys() and event["LogNotes"] == "DAG Node: C":
                slurm_cluster_id = event.cluster
            elif event.cluster == slurm_cluster_id and event.type == htcondor.JobEventType.EXECUTE:
                job_status = "RUNNING"
                jobs_running += 1
                if job_started_time is None:
                    job_started_time = datetime.fromtimestamp(event.timestamp)
            elif event.cluster == slurm_cluster_id and event.type == htcondor.JobEventType.JOB_TERMINATED:
                jobs_running -= 1
                if jobs_running == 0:
                    job_status = "COMPLETE"
            elif event.type == htcondor.JobEventType.JOB_HELD or event.type == htcondor.JobEventType.EXECUTABLE_ERROR:
                job_status = "ERROR"

        # Calculate how long job has been in its current state
        current_time = datetime.now()
        time_diff = None
        if job_status == "PROVISIONING REQUEST PENDING":
            time_diff = current_time - provisioner_job_submitted_time
        elif job_status == "RUNNING":
            time_diff = current_time - job_started_time

        # Now that we have all the information we want, display it
        if job_status == "COMPLETED":
            logger.info("Job has completed")
        else:
            if job_status == "PROVISIONING REQUEST PENDING":
                logger.info(f"Job is waiting for {resource_type.upper()} to provision pending request", end='')
            else:
                info_str = f"Job is {job_status}"
                if time_diff is not None:
                    info_str = f"{info_str} since {
                        round(time_diff.seconds/60)}m{(time_diff.seconds % 60)}s"
                logger.info(info_str)

    else:
        raise ValueError(f"Error: The 'job status' command does not support {resource_type} resources.")



import typer
import os
import traceback
import htcondor
import json
import re

from pathlib import Path
from typing_extensions import Annotated
from typing import List, Optional
from enum import Enum
from datetime import datetime
from datetime import timedelta

app = typer.Typer()


class attributes(str, Enum):
    GLIDEIN_ResourceName = "GLIDEIN_ResourceName"
    Memory = "Memory"


def convert_dhms_to_seconds(job_summary, key):
    """
    Converts a string in the format "0+00:00:00" to the number of seconds
    """
    # check if already in seconds
    if isinstance(job_summary.get(key, ""), int):
        return
    time = job_summary.get(key, "0+00:00:00")
    # convert time to seconds
    days, time_str = time.split("+")
    hours, minutes, seconds = map(int, time_str.split(":"))
    time_seconds = int(days)*24*60*60 + int(hours)*60 * \
        60 + int(minutes)*60 + int(seconds)
    job_summary[key] = time_seconds


def get_max_column_length(job_summaries, key):
    """
    Gets the length of the longest value for the specified key
    """
    max_length = len(key)
    for job_summary in job_summaries.values():
        length = len(str(job_summary.get(key, "")))
        if length > max_length:
            max_length = length
    return max_length + 2


def convert_seconds_to_dhms(seconds):
    """
    Converts a number of seconds to a string in the format "0+00:00:00"
    """
    days = seconds // (24*60*60)
    seconds = seconds % (24*60*60)
    hours = seconds // (60*60)
    seconds = seconds % (60*60)
    minutes = seconds // 60
    seconds = seconds % 60
    return f"{days}+{hours:02d}:{minutes:02d}:{seconds:02d}"


def get_job_summaries(log_file, group_by):
    """
    Gets a dictionary of job summaries from the specified log file
    """
    # dictionary to store summary of events for each job
    job_summaries = {}
    # definetly need to refactor this or test to make sure most use cases work like the three end cases should be termination, abort, and remove not evict like it is now
    job_event_log = htcondor.JobEventLog(str(log_file))
    for event in job_event_log.events(0):
        job_id = str(event.cluster) + "." + str(event.proc)
        job_summary = job_summaries.get(job_id)
        if job_summary is None:
            job_summary = {}
            job_summary["Submitted"] = False
            job_summary["Evictions"] = 0
            job_summary["Executed"] = False
            job_summary["starts"] = 0
            job_summary["Job ID"] = job_id
            job_summaries[job_id] = job_summary
        if event.type == htcondor.JobEventType.SUBMIT:
            ip_address = re.search(
                r"\d+\.\d+\.\d+\.\d+", event.get("SubmitHost")).group()
            job_summary["Submitted"] = True
            job_summary["Host"] = ip_address
        elif (event.type == htcondor.JobEventType.JOB_ABORTED or event.type == htcondor.JobEventType.JOB_EVICTED) and job_summary.get("Executed", False) is False or job_summary["Submitted"] is False:
            # remove job from dictionary if it was aborted before being executed or not submitted
            job_summaries.pop(job_id)
        elif event.type == htcondor.JobEventType.EXECUTE:
            # get hostname if possible
            if event.get("SlotName") is not None:
                hostname = event.get("SlotName")
                job_summary["Host"] = hostname
            # get grouping information, if any
            if group_by is not None and event.get("ExecuteProps") is not None:
                job_summary[group_by] = event.get(
                    "ExecuteProps").get(group_by)
            # get last execution time
            start_time = datetime.strptime(
                event.get("EventTime"), "%Y-%m-%dT%H:%M:%S").strftime("%-m/%-d %H:%M")
            job_summary["Start Time"] = start_time
            job_summary["last_exec_time"] = start_time
            # set boolean that job has been executed
            job_summary["Executed"] = True
            job_summary["starts"] = job_summary.get("starts", 0) + 1
        elif event.type == htcondor.JobEventType.JOB_EVICTED or event.type == htcondor.JobEventType.JOB_TERMINATED:
            # get good time and cpu usage
            cpu_usage_key = "RunRemoteUsage" if event.type == htcondor.JobEventType.JOB_EVICTED else "TotalRemoteUsage"
            cpu_usage = str(event.get(cpu_usage_key))
            user_time = re.search(
                r"Usr (\d+) (\d+):(\d+):(\d+),", cpu_usage)
            sys_time = re.search(r"Sys (\d+) (\d+):(\d+):(\d+)", cpu_usage)
            user_days, user_hours, user_minutes, user_seconds = user_time.group(
                1, 2, 3, 4)
            sys_days, sys_hours, sys_minutes, sys_seconds = sys_time.group(
                1, 2, 3, 4)
            total_days = int(user_days) + int(sys_days)
            total_hours = int(user_hours) + int(sys_hours)
            total_minutes = int(user_minutes) + int(sys_minutes)
            total_seconds = int(user_seconds) + int(sys_seconds)
            cpu_usage_formatted = f"{total_days}+{total_hours:02d}:{total_minutes:02d}:{total_seconds:02d}"
            job_summary["CPU Usage"] = cpu_usage_formatted
            if event.type == htcondor.JobEventType.JOB_EVICTED:
                job_summary["Evictions"] = job_summary.get(
                    "Evictions", 0) + 1
                job_summary["last_exec_time"] = datetime.strptime(
                    event.get("EventTime"), "%Y-%m-%dT%H:%M:%S").strftime("%-m/%-d %H:%M")
            job_summary["Evict Time"] = datetime.strptime(
                event.get("EventTime"), "%Y-%m-%dT%H:%M:%S").strftime("%-m/%-d %H:%M")
            delta = datetime.strptime(
                job_summary["Evict Time"], "%m/%d %H:%M") - datetime.strptime(job_summary["Start Time"], "%m/%d %H:%M")
            wall_time = f"{delta.days}+{delta.seconds // 3600:02d}:{(delta.seconds//60) % 60:02d}:{delta.seconds % 60:02d}"
            job_summary["Wall Time"] = wall_time
            # get total execution time for calculating good time
            event_time = datetime.strptime(
                event.get("EventTime"), "%Y-%m-%dT%H:%M:%S").strftime("%-m/%-d %H:%M")
            delta = datetime.strptime(
                event_time, "%m/%d %H:%M") - datetime.strptime(job_summary["last_exec_time"], "%m/%d %H:%M")
            good_time = f"{delta.days}+{delta.seconds // 3600:02d}:{(delta.seconds//60) % 60:02d}:{delta.seconds % 60:02d}"
            job_summary["Good Time"] = good_time
            # get Return Value
            job_summary["Return Value"] = event.get("ReturnValue")
    return job_summaries


def output_job_summaries(job_summaries, group_by, options):
    """
    Outputs the specified job summaries
    """
    # will probably need to refactor this to be more efficient for other group_by options
    if group_by is not None:
        if group_by == "GLIDEIN_ResourceName":
            # get total stats for each site
            site_stats = {}
            for job_id, job_summary in job_summaries.items():
                site = job_summary.get(group_by, "")
                if site not in site_stats:
                    site_stats[site] = {"Evictions": 0, "Wall Time": 0,
                                        "Good Time": 0, "CPU Usage": 0, "Successes": 0, "Failures": 0}
                site_stats[site]["Evictions"] += int(
                    job_summary.get("Evictions", 0))
                # get wall time and convert to time format for adding
                convert_dhms_to_seconds(job_summary, "Wall Time")
                site_stats[site]["Wall Time"] += job_summary.get(
                    "Wall Time", 0)
                # get good time and convert to time format for adding
                convert_dhms_to_seconds(job_summary, "Good Time")
                site_stats[site]["Good Time"] += job_summary.get(
                    "Good Time", 0)
                # get cpu usage and convert to seconds for adding
                convert_dhms_to_seconds(job_summary, "CPU Usage")
                site_stats[site]["CPU Usage"] += job_summary.get(
                    "CPU Usage", 0)
                # count number of times any job started at site
                site_stats[site]["Starts"] = site_stats[site].get(
                    "Starts", 0) + job_summary.get("starts")
                # count number of times any job ended at site with return value 0
                if job_summary.get("Return Value") == 0:
                    site_stats[site]["Successes"] = site_stats[site].get(
                        "Successes") + 1
                elif job_summary.get("Return Value") is None:
                    continue
                elif job_summary.get("Return Value") != 0:
                    site_stats[site]["Failures"] = site_stats[site].get(
                        "Failures") + 1
            # convert wall time, good time, and cpu usage to time format
            for site, stats in site_stats.items():
                site_stats[site]["Wall Time"] = convert_seconds_to_dhms(
                    stats["Wall Time"])
                site_stats[site]["Good Time"] = convert_seconds_to_dhms(
                    stats["Good Time"])
                site_stats[site]["CPU Usage"] = convert_seconds_to_dhms(
                    stats["CPU Usage"])
            # if json output, print json
            if options.get("json", True):
                print(json.dumps(site_stats, indent=4))
            elif options.get("csv", True):
                # print header
                print(
                    "Site,Evictions,Wall Time,Good Time,CPU Usage,Job Starts,Job Successes,Job Failures")
                # print stats for each site
                for site, stats in site_stats.items():
                    if site:
                        print(
                            f"{site},{stats['Evictions']},{stats['Wall Time']},{stats['Good Time']},{stats['CPU Usage']},{stats['Starts']},{stats.get('Successes', '')},{stats.get('Failures', '')}")
            else:
                # print header
                print(f"{'Site':<20} {'Evictions':<12} {'Wall Time':<12} {'Good Time':<12} {'CPU Usage':<12} {'Job Starts':<12} {'Job Successes':<14} {'Job Failures':<12}")
                # print stats for each site
                for site, stats in site_stats.items():
                    if site:
                        print(
                            f"{site:<20} {stats['Evictions']:<12} {stats['Wall Time']:<12} {stats['Good Time']:<12} {stats['CPU Usage']:<12} {stats['Starts']:<12} {stats.get('Successes', ''):<14} {stats.get('Failures', ''):<12}")
    elif options.get("json", True) or options.get("csv", True):
        # add event summaries as objects to list for JSON output
        json_output = []
        for job_id, job_summary in job_summaries.items():
            # convert cpu usage, wall time, and good time to seconds
            convert_dhms_to_seconds(job_summary, "CPU Usage")
            convert_dhms_to_seconds(job_summary, "Wall Time")
            convert_dhms_to_seconds(job_summary, "Good Time")
            json_output.append({"Job ID": job_id, **job_summary})
        if options.get("csv", True):
            print(
                "Job ID,Host,Start Time,Evict Time,Evictions,Wall Time,Good Time,CPU Usage")
            for job_id, job_summary in job_summaries.items():
                print(f"{job_id},{job_summary.get('Host', '')},{job_summary.get('Start Time', '')},{job_summary.get('Evict Time', '')},{job_summary.get('Evictions', '')},{job_summary.get('Wall Time', '')},{job_summary.get('Good Time', '')},{job_summary.get('CPU Usage', '')}")
        else:
            print(json.dumps(json_output, indent=4))
    else:
        # determine maximum length of each column
        max_job_id_len = get_max_column_length(job_summaries, "Job ID")
        max_host_len = get_max_column_length(job_summaries, "Host")
        max_start_time_len = get_max_column_length(job_summaries, "Start Time")
        max_evict_time_len = get_max_column_length(job_summaries, "Evict Time")
        max_evictions_len = get_max_column_length(job_summaries, "Evictions")
        max_time_format = 13

        # print header with dynamic column spacing
        header = f"{'Job':<{max_job_id_len}} {'Host':<{max_host_len}} {'Start Time':<{max_start_time_len}} {'Evict Time':<{max_evict_time_len}} {'Evictions':<{max_evictions_len}} {'Wall Time':<{max_time_format}} {'Good Time':<{max_time_format}} {'CPU Usage':<{max_time_format}}  "
        print(header)
        i = 1
        # print data for each job with dynamic column spacing
        for job_id, data in job_summaries.items():
            if i % 50 == 0:
                print(header)
            host = data.get("Host", "")
            start_time = data.get("Start Time", "")
            evict_time = data.get("Evict Time", "")
            evictions = data.get("Evictions", "")
            wall_time = data.get("Wall Time", "")
            good_time = data.get("Good Time", "")
            cpu_usage = data.get("CPU Usage", "")
            print(f"{job_id:<{max_job_id_len}} {host:<{max_host_len}} {start_time:<{max_start_time_len}} {evict_time:<{max_evict_time_len}} {evictions:<{max_evictions_len}} {wall_time:<{max_time_format}} {good_time:<{max_time_format}} {cpu_usage:<{max_time_format}} ")
            i += 1


@app.command()
def read(
    log_files: Annotated[List[str], typer.Argument(..., help="Log file or space separated list of log files")],
    json: Annotated[bool, typer.Option(help="Output in JSON format")] = False,
    csv: Annotated[bool, typer.Option(help="Output in CSV format")] = False,
    group: Annotated[Optional[attributes], typer.Option(
        help="Group by this attribute")] = None
):
    """
    Reads an eventlog
    """
    job_summaries_across_files = {}  # Stores summary of events for all jobs across files
    for file_path in log_files:
        # Ensure each specified log file exists and is readable
        log_file = Path(file_path)
        if not log_file.exists():
            raise FileNotFoundError(f"Could not find file: {str(log_file)}")
        if os.access(log_file, os.R_OK) is False:
            raise PermissionError(f"Could not access file: {str(log_file)}")
        try:
            current_file_job_summaries = get_job_summaries(log_file, group)
            # Merge current file's job summaries into the collective summary
            for job_id, job_summary in current_file_job_summaries.items():
                if job_id not in job_summaries_across_files:
                    job_summaries_across_files[job_id] = job_summary
                else:
                    # Update existing job summary with new information from current file
                    job_summaries_across_files[job_id].update(job_summary)
        except Exception as error:
            traceback.print_exc()
            return
    # Output aggregated job summaries
    options = {"json": json, "csv": csv}
    output_job_summaries(job_summaries_across_files, group, options)

@app.command()
def follow(
    log_file: Annotated[str, typer.Argument(..., help="Log file")],
    json: Annotated[bool, typer.Option(help="Output in JSON format")] = False,
    csv: Annotated[bool, typer.Option(help="Output in CSV format")] = False,
    group: Annotated[Optional[attributes], typer.Option(help="Group by this attribute")] = None
    ):
    """
    Follows an eventlog
    """
    # make sure the specified log file exists and is readable
    log_file = Path(log_file)
    if not log_file.exists():
        raise FileNotFoundError(f"Could not find file: {str(log_file)}")
    if os.access(log_file, os.R_OK) is False:
        raise PermissionError(f"Could not access file: {str(log_file)}")
    try:
        # output job summary when log file is updated
        last_size = log_file.stat().st_size
        options = {"json": json, "csv": csv}
        output_job_summaries(get_job_summaries(
            log_file, group), group, options)
        while True:
            # check if the log file has changed
            current_size = log_file.stat().st_size
            if current_size != last_size:
                # get event summaries
                job_summaries = get_job_summaries(log_file, group)
                # output event summaries
                output_job_summaries(job_summaries, group, options)
                # update last size
                last_size = current_size
    except KeyboardInterrupt:
        # exit gracefully if user presses Ctrl+C
        print(" Exiting...")
        pass

if __name__ == "__main__":
    app()


class DAGMan:
    """
    A :class:`DAGMan` holds internal operations related to DAGMan jobs
    """


    @staticmethod
    def get_files(dagman_id):
        """
        Retrieve the filenames of a DAGs output and event logs based on
        DAGMan cluster id
        """

        dag, iwd, log, out = None, None, None, None

        # Get schedd
        schedd = htcondor.Schedd()

        env = schedd.query(
            constraint=f"ClusterId == {dagman_id}",
            projection=["Env", "Iwd"],
        )

        if env:
            iwd = env[0]["Iwd"]
            env = dict(item.split("=", 1) for item in env[0]["Env"].split(";"))
            out = Path(iwd) / Path(os.path.split(env["_CONDOR_DAGMAN_LOG"])[1])
            log = Path(str(out).replace(".dagman.out", ".nodes.log"))
            dag = Path(str(out).replace(".dagman.out", ""))

        return str(dag), str(out), str(log)


    @staticmethod
    def write_slurm_dag(jobfile, runtime, email):

        sendmail_sh = "#!/bin/sh\n"
        if email is not None:
            sendmail_sh += f"\necho -e '$2' | mail -v -s '$1' {email}\n"

        with open(TMP_DIR / "sendmail.sh", "w") as sendmail_sh_file:
            sendmail_sh_file.write(sendmail_sh)
        st = os.stat(TMP_DIR / "sendmail.sh")
        os.chmod(TMP_DIR / "sendmail.sh", st.st_mode | stat.S_IEXEC)

        slurm_config = "DAGMAN_USE_DIRECT_SUBMIT = True\nDAGMAN_USE_STRICT = 0\n"
        with open(TMP_DIR / "slurm_submit.config", "w") as slurm_config_file:
            slurm_config_file.write(slurm_config)

        slurm_dag = f"""JOB A {{
    executable = sendmail.sh
    arguments = \\\"Job submitted to run on Slurm\\\" \\\"Your job ({jobfile}) has been submitted to run on a Slurm resource\\\"
    universe = local
    output = job-A-email.$(cluster).$(process).out
    request_disk = 10M
}}
JOB B {{
    executable = /home/chtcshare/hobblein/hobblein_remote.sh
    universe = grid
    grid_resource = batch slurm hpclogin1.chtc.wisc.edu
    transfer_executable = false
    output = job-B.$(Cluster).$(Process).out
    error = job-B.$(Cluster).$(Process).err
    log = job-B.$(Cluster).$(Process).log
    annex_runtime = {runtime}
    annex_node_count = 1
    annex_name = {getpass.getuser()}-annex
    annex_user = {getpass.getuser()}
    # args: <node count> <run time> <annex name> <user>
    arguments = $(annex_node_count) $(annex_runtime) $(annex_name) $(annex_user)
    +NodeNumber = $(annex_node_count)
    +BatchRuntime = $(annex_runtime)
    request_disk = 30
    notification = NEVER
}}
JOB C {jobfile} DIR {os.getcwd()}
JOB D {{
    executable = sendmail.sh
    arguments = \\\"Job completed run on Slurm\\\" \\\"Your job ({jobfile}) has completed running on a Slurm resource\\\"
    universe = local
    output = job-D-email.$(cluster).$(process).out
    request_disk = 10M
}}

PARENT A CHILD B C
PARENT B C CHILD D

CONFIG slurm_submit.config

VARS C Requirements="(Facility == \\\"CHTC_Slurm\\\")"
VARS C +MayUseSlurm="True"
VARS C +WantFlocking="True"
"""

        with open(TMP_DIR / "slurm_submit.dag", "w") as dag_file:
            dag_file.write(slurm_dag)


    @staticmethod
    def write_ec2_dag(jobfile, runtime, email):

        sendmail_sh = "#!/bin/sh\n"
        if email is not None:
            sendmail_sh += f"\necho -e '$2' | mail -v -s '$1' {email}\n"

        with open(TMP_DIR / "sendmail.sh", "w") as sendmail_sh_file:
            sendmail_sh_file.write(sendmail_sh)
        st = os.stat(TMP_DIR / "sendmail.sh")
        os.chmod(TMP_DIR / "sendmail.sh", st.st_mode | stat.S_IEXEC)

        ec2_annex_sh = f"""#!/bin/sh

yes | /usr/bin/condor_annex -count 1 -duration $1 -annex-name EC2Annex-{int(time.time())}
"""
        with open(TMP_DIR / "ec2_annex.sh", "w") as ec2_annex_sh_file:
            ec2_annex_sh_file.write(ec2_annex_sh)
        st = os.stat(TMP_DIR / "ec2_annex.sh")
        os.chmod(TMP_DIR / "ec2_annex.sh", st.st_mode | stat.S_IEXEC)

        ec2_config = "DAGMAN_USE_DIRECT_SUBMIT = True\nDAGMAN_USE_STRICT = 0\n"
        with open(TMP_DIR / "ec2_submit.config", "w") as ec2_config_file:
            ec2_config_file.write(ec2_config)

        ec2_dag = f"""JOB A {{
    executable = sendmail.sh
    arguments = \\\"Job submitted to run on EC2\\\" \\\"Your job ({jobfile}) has been submitted to run on a EC2 resource\\\"
    universe = local
    output = job-A-email.$(cluster).$(process).out
    request_disk = 10M
}}
JOB B {{
    executable = ec2_annex.sh
    arguments = {runtime}
    output = job-B-ec2_annex.$(Cluster).$(Process).out
    error = job-B-ec2_annex.$(Cluster).$(Process).err
    log = ec2_annex.log
    universe = local
    request_disk = 10M
}}
JOB C {jobfile} DIR {os.getcwd()}
JOB D {{
    executable = sendmail.sh
    arguments = \\\"Job completed run on EC2\\\" \\\"Your job ({jobfile}) has completed running on a EC2 resource\\\"
    universe = local
    output = job-D-email.$(cluster).$(process).out
    request_disk = 10M
}}

PARENT A CHILD B C
PARENT B C CHILD D

CONFIG ec2_submit.config

VARS C Requirements="(EC2InstanceID =!= undefined) && (TRUE || TARGET.OpSysMajorVer)"
VARS C +MayUseAWS="True"
VARS C +WantFlocking="True"
"""

        with open(TMP_DIR / "ec2_submit.dag", "w") as dag_file:
            dag_file.write(ec2_dag)


import typer
import htcondor

JSM_HTC_DAG_SUBMIT = 4

app = typer.Typer()

@app.command()
def submit():
    """
    Submits a job when given a submit file
    """
    typer.echo("Submitting a job")

@app.command()
def status():
    """
    Shows current status of a DAG when given a DAG id
    """
    typer.echo("Checking the status of a job")


import typer
from typer_htcondor import eventlog
from typer_htcondor import job
from typer_htcondor import dag
app = typer.Typer()
app.add_typer(eventlog.app, name="eventlog", help="Interacts with eventlogs")
app.add_typer(job.app, name="job", help="Run operations on HTCondor jobs")
app.add_typer(dag.app, name="dag", help="Run operations on HTCondor DAGs")

if __name__ == "__main__":
    app()
